

(function () {

  var vCompare = function (v1, v2, options) {
    var lexicographical = options && options.lexicographical,
        zeroExtend = options && options.zeroExtend,
        v1parts = v1.split('.'),
        v2parts = v2.split('.');

    function isValidPart(x) {
        return (lexicographical ? /^\d+[A-Za-z]*$/ : /^\d+$/).test(x);
    }

    if (!v1parts.every(isValidPart) || !v2parts.every(isValidPart)) {
        return NaN;
    }

    if (zeroExtend) {
        while (v1parts.length < v2parts.length) v1parts.push('0');
        while (v2parts.length < v1parts.length) v2parts.push('0');
    }

    if (!lexicographical) {
        v1parts = v1parts.map(Number);
        v2parts = v2parts.map(Number);
    }

    for (var i = 0; i < v1parts.length; ++i) {
        if (v2parts.length == i) {
            return 1;
        }

        if (v1parts[i] == v2parts[i]) {
            continue;
        }
        else if (v1parts[i] > v2parts[i]) {
            return 1;
        }
        else {
            return -1;
        }
    }

    if (v1parts.length != v2parts.length) {
        return -1;
    }

    return 0;
  };

  var getMobileOperatingSystem = function() {
    var userAgent = navigator.userAgent || navigator.vendor || window.opera;

        // Windows Phone must come first because its UA also contains 'Android'
      if (/windows phone/i.test(userAgent)) {
          return 'WinCE';
      }

      if (/android/i.test(userAgent)) {
          return 'Android';
      }

      // iOS detection from: http://stackoverflow.com/a/9039885/177710
      if (/iPad|iPhone|iPod/.test(userAgent) && !window.MSStream) {
          return 'iOS';
      }

      return null;
  }

  var convertToRedirectablePath = function (url) {

    // to convert 'file:///aaa/bbb/ccc/index.html' -> '/aaa/bbb/ccc/index.html'

    // for ios      it will redirect to 'http://localhost:8080/aaa/bbb/ccc/index.html'; (wkwebview requires a http server)
    // for android  it will redirect to 'file:///aaa/bbb/ccc/index.html'

    var ret = url.indexOf('file:///') === 0 ? url.substr(('file://').length) : url;
    return ret;

    // useful in dev mode
    // return location.href.indexOf('http://wv.local-dev.mkyice.cn:20001') === 0 ? 'http://localhost:8080' + ret : ret;
  };

//var _DEFAULT_VERSIONSERVERBASEURL = 'http://wv.mkyice.cn/mcwvavc';
//var _MAIN_SERVERBASEURL           = 'http://wv.mkyice.cn';

  var _DEFAULT_VERSIONSERVERBASEURL = 'http://wv.mkyice.menco.cn/mcwvavc';
  var _MAIN_SERVERBASEURL           = 'http://wv.mkyice.menco.cn';

  var appPath  = null;
  var dataPath = null;
  var wvcPath  = null;

  var mode                 = null;

  var versionServerBaseUrl = null;

  var current_appname      = null;

  var current_nv           = null;
  var current_nvp          = null;

  var current_wvcv         = null;

  var current_platform     = null;


  var webviewInBackground = false;

  window.AppVersionManager = {

    ___dev_removeAll: function () {

      window.resolveLocalFileSystemURL(
        dataPath,
        function (baseEntry) {
          baseEntry.getDirectory('wvc', {}, function (wvcEntry) {
            wvcEntry.removeRecursively();
          })
        }
      )

    },

    ___dev_inspect: function (dirPath, success, fail) {

      window.resolveLocalFileSystemURL(
        dirPath,
        function (dirEntry) {
          dirEntry.createReader().readEntries(
            function (entries) {
              console.log(entries);
              success && success(entries);
            },
            function (error) {
              fail && fail(error);
            }
          );
        }
      )

    },

    MAIN_SERVERBASEURL: _MAIN_SERVERBASEURL,

    versionCompare: function(v1, v2) {

      var parts_v1 = v1.split('-');
      var parts_v2 = v2.split('-');

      return vCompare(parts_v1[1], parts_v2[1]);

    },

    dataPath: function () { return dataPath; },
    wvcPath:  function () { return wvcPath;  },

    versionInfoUrl: function () { return versionServerBaseUrl + '/n/' + current_nvp },

    current_nvp:      function () { return current_nvp; },
    current_nv:       function () { return current_nv; },
    current_wvcv:     function () { return current_wvcv; },
    current_platform: function () { return current_platform; },

    current_uav:      function () { return current_wvcv + '_' + current_nvp; },

    initAppInit:  function (data, success, fail) {

      var self = this;

      this._init(
        data,
        function () {
          self._initialPack_unpack(
            function (unpack) {
              if (unpack.status === 'ok')
                success();
              else
                fail();
            }
          );
        },
        fail
      );

    },

    appInit: function () {

      this.wvc_latest(
        function (wvcv_latest) {
          location.href = convertToRedirectablePath(wvcv_latest.nativeURL + '/appStart/appStart.html');
        }
      );

    },

    initAppStart: function (data, success, fail) {

      var self = this;

      this._init(
        data,
        function () {
          success();
        },
        fail
      );

    },

    initAppRun: function (data, success, fail) {

      var self = this;

      this._init(
        data,
        function () {
          success();
        },
        fail
      );

    },

    _init: function (data, success, fail) {

      mode                 = data.mode;
      versionServerBaseUrl = data.versionServerBaseUrl ? data.versionServerBaseUrl : _DEFAULT_VERSIONSERVERBASEURL;

      current_wvcv         = data.current_wvcv;

      var self = this;

      document.addEventListener('deviceready', function () {

        appPath  = cordova.file.applicationDirectory;
        dataPath = cordova.file.dataDirectory;
        wvcPath  = cordova.file.dataDirectory + 'wvc';

        Promise.all(
          [
            cordova.getAppVersion.getVersionNumber(),
            cordova.getAppVersion.getAppName()
          ]
        ).then(
          function (values) {
            var version = values[0];
            var appname = values[1];

            current_appname  = appname;

            current_platform = getMobileOperatingSystem().toLowerCase();

            current_nv       = 'nv-' + version;
            current_nvp      = 'nv-' + version + '_' + current_platform;

            window.resolveLocalFileSystemURL(dataPath, function (baseEntry) {

              baseEntry.getDirectory('wvc', { create: true }, success, fail);

            });


          }
        );

      });


    },

    _initialPack_read: function (success) {

      var xhr = new XMLHttpRequest;

      xhr.open('GET', './appVersion.json');

      xhr.onload = function () {
        var data = JSON.parse(xhr.responseText);
        success(data);
      };

      xhr.send();

    },

    _initialPack_unpack: function (unpackCallback) {

      var self = this;

      var packedBefore = function () {
        unpackCallback(
          {
            status: 'ok',
            type: 'packedBefore'
          }
        );
      };

      var packedOK = function () {
        unpackCallback(
          {
            status: 'ok',
            type: 'packedOK'
          }
        );
      };

      var tryUnpack = function (data) {
        var initialPackPath = appPath + 'www/' + data['wvc']['file'];

        var _transferInitialPack = function (initialPackPath, success, fail) {

          window.resolveLocalFileSystemURL(
            wvcPath,
            function (dirEntry) {

              window.resolveLocalFileSystemURL(
                initialPackPath,
                function (entry) {
                  entry.copyTo(dirEntry, entry.name, success, fail);
                },
                fail
              );

            },
            fail
          );

        };

        _transferInitialPack(
          initialPackPath,
          function () { self.wvc_unpack(data['wvc']['wvcv'], packedOK); }
        );

      };

      this._initialPack_read(
        function (data) {
          self.wvc_isOK( data['wvc']['wvcv'], packedBefore, function () { tryUnpack(data); } );
        }
      );

    },


    wvc_getFSPath: function (wvcv) {
      return wvcPath + '/' + wvcv;
    },

    wvc_loadFSEntries: function (success, fail) {

      window.resolveLocalFileSystemURL(
        wvcPath,
        function (entry) {

          entry.createReader().readEntries(
            function (entries) {
              success(entries);
            },
            function (error) {
              fail(error);
            }
          );

        },
        function (error) {
          fail(error);
        }
      );

    },

    wvc_unpack: function (wvcv, success) {

      var zipURL = this.wvc_getFSPath(wvcv) + '.zip';

      window.resolveLocalFileSystemURL(wvcPath, function (wvcDirEntry) {

        wvcDirEntry.getDirectory(wvcv, {create: true}, function(dirEntry) {

          zip.unzip(zipURL, dirEntry.toURL(), function (code) {

            if (code === 0) {
              dirEntry.getFile('___DONE', {create: true});
              success();
            }

          });

        });

      });

    },

    wvc_isOK: function (wvcv, success, fail) {
      window.resolveLocalFileSystemURL(this.wvc_getFSPath(wvcv) + '/___DONE', success, fail);
    },


    wvc_getAll: function (success, fail) {

      var self = this;

      var isValidWVC = function (fEntry) {
        //todo check wvc_isOK
        return fEntry.isDirectory === true;
      };

      this.wvc_loadFSEntries(
        function (entries) {

          var results = [];
          for (var i = 0; i < entries.length; i++)
            if (isValidWVC(entries[i]))
              results.push(
                {
                  wvcv:      entries[i].name,
                  nativeURL: entries[i].nativeURL.replace(/\/$/,"")
                }
              );

          results.sort(function (v1, v2) {
            return -self.versionCompare(v1.wvcv, v2.wvcv);
          });

          success(results);

        },
        fail
      );

    },

    wvc_latest: function (success, fail) {

      this.wvc_getAll(
        function (wvcvs) {
          success(wvcvs.length > 0 ? wvcvs[0] : null);
        },
        fail
      );

    },


    wvc_update: function (wvc_update, success, fail) {

      var self = this;

      var zipURL = this.wvc_getFSPath(wvc_update.wvcv) + '.zip';

      (new FileTransfer).download(
        wvc_update.link,
        zipURL,
        function (entry) {

          self.wvc_unpack(wvc_update.wvcv, success);

        },
        fail,
        null,
        {

        }
      );

    },


    readVersionInfo: function (success) {

      var self = this;

      var xhr = new XMLHttpRequest;

      xhr.open('GET', this.versionInfoUrl());
      if (localStorage.getItem('mId'))
        xhr.setRequestHeader('MENCO-ASSESSMENT-MID', localStorage.getItem('mId'));
      xhr.setRequestHeader('MENCO-ASSESSMENT-UAV', self.current_uav());

      xhr.onload = function () {
        var data = JSON.parse(xhr.responseText);

        success(data);
      };

      xhr.send();

    },


    appStart: function () {

      var self = this;

      self.readVersionInfo(
        function (data) {

          var is_N_Supported = data.n_supports.indexOf(current_nvp) >= 0;

          if (is_N_Supported) {

            self.wvc_latest(
              function (wvcv_latest) {

                is_WVC_Supported = wvcv_latest && data.wvc_supports.indexOf(wvcv_latest.wvcv) >= 0;

                if (is_WVC_Supported) {
                  //goto local
                  location.href = convertToRedirectablePath(wvcv_latest.nativeURL + '/index.html');
                }
                else {

                  //try update
                  self.wvc_update(data.wvc_update, function () {

                    location.href = convertToRedirectablePath(self.wvc_getFSPath(data.wvc_update.wvcv) + '/index.html');

                  });

                }

              }
            );

          }
          else {

            if (data.n_update) {

              if (current_platform === 'android') {

                var link     = data.n_update.link;
                var filename = current_appname + '-' + data.n_update.nv + '.apk';

                navigator.notification.confirm(
                  'app版本需要升级',
                  function (i) {
                    if (i === 1) {

                      //todo add downloading... prompt

                      window.MencoAssessmentCordovaPluginDownloadManager.download(
                        {
                          url: link,
                          path: filename,
                          dirType: 'application'
                        },
                        function (downloadedFile) {

                          cordova.plugins.fileOpener2.open(
                            downloadedFile,
                            'application/vnd.android.package-archive'
                          );

                          navigator.app.exitApp();

                        },
                        function (message) {

                          navigator.notification.alert(
                            'app升级失败',
                            function () {
                              //todo retry
                              cordova.InAppBrowser.open(data.n_update.pageLink, '_system', '');
                              navigator.app.exitApp();
                            },
                            '系统',
                            '确定'
                          );

                        }
                      );
                    }
                    else {
                      cordova.InAppBrowser.open(data.n_update.pageLink, '_system', '');
                      navigator.app.exitApp();
                    }

                  },
                  '系统',
                  ['升级', '取消']
                );

              }
              else if (current_platform === 'ios') {

                var alert_ios_version_upgrade = function () {
                  navigator.notification.alert(
                    'app版本太低,请先升级！',
                    function () {
                      cordova.InAppBrowser.open(data.n_update.pageLink, '_system', '');
                      alert_ios_version_upgrade();
                    },
                    '系统',
                    '确定'
                  );
                };

                alert_ios_version_upgrade();

              }

            }
            else {

              var alert_version_not_support = function () {

                navigator.notification.alert(
                  'app版本无法支持,请先升级！',
                  function () {
                    if (current_platform === 'android')
                      navigator.app.exitApp();
                    else
                      alert_version_not_support();
                  },
                  '系统',
                  '确定'
                );

              };

              alert_version_not_support();

            }

          }

        }
      );

    },


    appRun: function () {

      // document.addEventListener('pause', function () {
      //   webviewInBackground = true;
      // });

      // document.addEventListener('resume', function () {
      //   webviewInBackground = false;
      // });

      cordova.plugins.backgroundMode.setDefaults({
        title: current_appname,
        text:  '',
        silent: true
      });

      cordova.plugins.backgroundMode.enable();

      cordova.plugins.backgroundMode.on('activate', function () {
        webviewInBackground = true;
      });

      cordova.plugins.backgroundMode.on('deactivate', function () {
        webviewInBackground = false;
      });

      var self = this;

      var isChecking = false;
      var handleUpdate = function () {

        if (!webviewInBackground)
          return;

        if (isChecking)
          return;

        isChecking = true;

        self.checkForUpdate(
          function () {
            console.log('finish');
            isChecking = false;

            // 15 mins later check again
            if (webviewInBackground)
              setTimeout(handleUpdate, 15 * 1000 * 60);
              // setTimeout(handleUpdate, 1000);
          }
        );

      };

      // document.addEventListener('pause', handleUpdate);

      cordova.plugins.backgroundMode.on('activate', handleUpdate);

    },

    checkForUpdate: function (finish) {

      var self = this;

      self.readVersionInfo(
        function (data) {

          var is_N_Supported = data.n_supports.indexOf(current_nvp) >= 0;

          if (is_N_Supported) {

            self.wvc_latest(
              function (wvcv_latest) {

                is_WVC_Supported = wvcv_latest && data.wvc_supports.indexOf(wvcv_latest.wvcv) >= 0;

                if (is_WVC_Supported) {

                  //background update
                  if (data.wvc_update && self.versionCompare(wvcv_latest.wvcv, data.wvc_update.wvcv) < 0) {

                    self.wvc_update(data.wvc_update, function () {

                      if (webviewInBackground)
                        location.href = convertToRedirectablePath(self.wvc_getFSPath(data.wvc_update.wvcv) + '/index.html');
                      else
                        finish();

                    }, finish);

                  }
                  else {

                    finish();

                  }

                }
                else {

                  if (data.wvc_update) {

                    self.wvc_update(data.wvc_update, function () {

                      if (webviewInBackground)
                        location.href = convertToRedirectablePath(self.wvc_getFSPath(data.wvc_update.wvcv) + '/index.html');
                      else
                        finish();

                    }, finish);

                  }
                  else {

                    finish();

                  }

                }

              }
            );

          }
          else {
            // native not support any more
            // do nothing
            finish();
          }

        }
      );

    }

  };

})();
